/****************************************************************************
 *                                                                          *
 * A small font library for the Empeg                                       *
 *                                                                          *
 * (c) 2001 Till Gerken, till@tantalo.net                                   *
 * This code is released under the GNU Public License.                      *
 * Parts are taken from Kim Salo, kim@remedy.fi                             *
 *                                                                          *
 * The VGA fonts have been copied from the plex86 VGA BIOS and are public   *
 * domain as per the file header.                                           *
 ****************************************************************************/

#include <sys/types.h>
#include <sys/stat.h>
#include <fcntl.h>
#include <unistd.h>
#include <sys/mman.h>
#include <sys/ioctl.h>

#include "font.h"
#include "vgafonts.h"

caddr_t font_dmap;
int font_fd;

unsigned char *font_bitmap = &vgafont8[0];
int font_x_size = 8;
int font_y_size = 8;

unsigned char font_display_buffer[FONT_DISPLAY_BYTES];

/*
 * draws a single pixel to the screen
 */

void font_set_pixel(unsigned char *buffer, int x, int y, unsigned char value)
{
	int buffer_position;
	int high_bit;
	unsigned char mask;
	
	buffer_position = (y * FONT_MAX_X) + x;
	
	high_bit = buffer_position & 1;
	
	buffer_position >>= 1;
	
	if(high_bit)
	{
		value <<= 4;
		mask = 0x0f;
	}
	else
	{
		mask = 0xf0;
	}
	
	buffer[buffer_position] = (buffer[buffer_position] & mask) | value;

}

/*
 * initialize font library, locks display
 */

int font_init()
{
	int i;

	/* open the display */
	font_fd = open("/dev/display", O_RDWR);
	if(font_fd == -1)
		return(1);

	/* map it to memory */
	font_dmap = mmap(0, 2048, PROT_READ | PROT_WRITE, MAP_SHARED, font_fd, 0);
	if (font_dmap == (caddr_t) -1)
		return(1);
	
	/* read in current display contents to our buffer */
	for(i = 0; i < FONT_DISPLAY_BYTES; i++)
		font_display_buffer[i] = font_dmap[i];

	return(0);
	
}

/*
 * deinitialize font library, frees display
 */

int font_deinit()
{

	if((munmap(font_dmap, 2048) == -1) || (close(font_fd) == -1))
		return(1);

}

/*
 * clears the screen
 */

int font_clear()
{
	int i;
	
	for(i = 0; i < FONT_DISPLAY_BYTES; i++)
	{
		font_display_buffer[i] = 0;
	}

	return(0);
	
}

/*
 * send current buffer contents to the display
 * turns the display on
 */

int font_blit()
{
	int i;

	/* copy all buffer contents to the mapped display */
	for(i = 0; i < FONT_DISPLAY_BYTES; i++)
	{
		font_dmap[i] = font_display_buffer[i];
	}

	FONT_BLIT(font_fd);
	FONT_DISPLAY(font_fd, 1);

	return(0);

}

/*
 * displays a character at the specified position
 */

int font_putch(int pos_x, int pos_y, char chr, char intensity)
{
	int x, y;
	
	/* check for invalid region */
	if(((pos_x + font_x_size) > FONT_MAX_X) || ((pos_y + font_y_size) > FONT_MAX_Y))
		return(1);
	
	/* check for invalid intensity */
	if(intensity > FONT_MAX_INTENSITY)
		return(1);
	
	/* now draw the character */
	for(y = 0; y < font_y_size; y++)
	{
		for(x = 0; x < font_x_size; x++)
		{
			font_set_pixel(font_display_buffer, x + pos_x, y + pos_y, (font_bitmap[chr * font_y_size + y] & (1 << (font_x_size - x))) ? intensity : 0);
		}
	}
	
	return(0);

}

/*
 * displays a string at the specified position
 */
int font_puts(int pos_x, int pos_y, char *str, char intensity)
{
	int i;
	
	for(i = 0; i < strlen(str); i++)
	{
		font_putch(pos_x, pos_y, str[i], intensity);
		pos_x += font_x_size;
		if(pos_x > FONT_MAX_X)
		{
			pos_y += font_y_size;
			pos_x = 0;
		}
	}
	
	return(0);
	
}

int main(void)
{
	
	if(font_init())
	{
		printf("Font library initialization failed\n");
		exit(1);
	}

	font_clear();
	font_puts(0, 0, "MatrixMatrixMatrixMatrixMatrixMatrixMatrixMatrixMatrixMatrixMatrixMatrix", 3);
	font_blit();
	
	if(font_deinit())
	{
		printf("Font deinitialization failed\n");
		exit(1);
	}
	
	return(0);
	
}
